#ifndef __SCANNER_RFC_H
#define __SCANNER_RFC_H

#ifndef __SCANNER_SERVER_H
#include "scanner_server.h"
#endif



class scanner_rfc : public scanner_server
{
private:

protected:
    /// This holds our list of read config fingerprints from file
    dllist<fp_conf> conffp;
    /// This holds our list of read rfc fingerprints from file
    dllist<fp_rfc> rfcfp;
    /// This holds the list of return fingerprints from file
    dllist<fp_return> returnfp;
    /// This is the pointer to the conf fingerprints from the last scan
    fp_conf * scanned_conf;
    /// This is the pointer to the rfc fingerprints from the last scan
    fp_rfc * scanned_rfc;
    /// This is the pointer to the return fingerprints from the last scan
    fp_return * scanned_return;

    const char ** cmd_list;

    virtual BOOL add_fp( fp& n_fp, fp_type f_type);
    
    virtual BOOL load_rfc_fps (void)=0;
    virtual BOOL load_return_fps (void)=0;
    virtual BOOL load_conf_fps (void)=0;
    virtual BOOL do_login (const mstring& username, const mstring& password)=0;

    uint32 cmd_number;

public:
    virtual ~scanner_rfc();
    /// We need a socket, the debuglevel, if beeing polite, connect & receive timeout
    scanner_rfc(small_socket& sk, int debug, bool pol, long ct, long rt) :
	    scanner_server(sk,debug, pol,ct, rt), scanned_conf(NULL), scanned_rfc(NULL), scanned_return(NULL),cmd_list(NULL),cmd_number(0){ };
/// returns the protocol we are scanning
inline virtual mstring protocol( void ) { return "RFC VIRTUAL"; }
/// does everything for initialization, returns false in case of error
virtual bool initialize( void );
/// finalize a scan (generate error values)
virtual void finalize( void );
/// returns all the fingerprints that will then be displayed
virtual dllist < dllist < fp > > get_fps( void );
/// returns the descriptions belonging to the fingerprints
virtual dllist< mstring > get_descs( void );
/// Does the real scan
virtual bool do_scan( const mstring& username, const mstring& password );

};

#endif

