/**
 * $Id: scanner_server.cpp,v 1.13 2002/11/25 23:33:35 plasmahh Exp $
 * $Revision: 1.13 $
 * $Author: plasmahh $
 * $Date: 2002/11/25 23:33:35 $
 */

#include "scanner_server.h"

mstring scanner_server::get_rfc_answer ( void )/*{{{*/
{
    // get answer string... search for last line, then search for %%%- like rfc answers... 
    // the last line is without the -, so return if found
    mstring answer="";
    mstring temp;
    mstring last;
    int nocount = 0; // counts number of empty answers
    int code,n_code;
    code = n_code = 0;
    // TODO what if we are disconnected ?
    while ( sock.Connected() )
    {
	temp = sock.Receive(10000,recv_timeout);
	answer += temp;
	if ( answer.length() < 4 ) 
	{
	    nocount++;
	    if ( nocount > 5 )
	    {
		sock.Disconnect();
		usleep (10000);
	    }
	    continue;
        }

        while ( temp.lsearch("\n") > 0 )
	{
	    last = temp;
	    temp = temp.substring(temp.lsearch("\n",0)+1,0);
	}
	n_code = strtoul(last,NULL,10);
	if ( code != 0 && n_code != code && n_code != 0)
	{
	    cout << "Possible multiple returncodes found. rerun wirh max. debuglevel, and inform author" << endl;
	}
	if ( code == 0)
	{
	    code = n_code;
	}
	if ( last.lsearch("-") != 4 && n_code != 0)
	{
	    break;
	}
    }
    return answer;
}/*}}}*/
//######## New Version of this load function !!
BOOL scanner_server::load_fp (const mstring& file, const mstring& identifier, fp_type fpt,BOOL check_leng)/*{{{*/
{

    mstring filename;
    filename.format ("%s/share/smtpmap/%s", _PREFIX, (const char*) file );
    mfile datei (filename);
    // Check if the file could be opened
    if (!datei.Open ())
    {
        cout << "Error opening file " << filename << endl;
        perror (NULL);
        return false;
    }

    mstring line;
    mstring version;
    mstring fp_line;

    fp *cfp = NULL;
    uint32 fps;
    uint32 lenge = 0;
    uint32 lines = 0;
    BOOL was = true; // false means last was a version, true means last was a fingerprint.
    BOOL ret = true;

    // As long as there is data, read it ;)
    while (!datei.Eof ())
    {
	line = get_next_line(datei);
        lines++; // because of this, we need a comment line to 
        if (line.length () == 0)
	    continue;		// In this case, it was only an empty line, so ignore...
        if (line.strip_all_whitespaces().matches("VERSION="))
	{
	    if (!was)		// if we have multiple lines, this was already set 
	    {
		printerror("Version line without a fingerprint found!",lines);
		ret = false;
	    }
	    was = false;
	    version = line.substring( 9,0 );
	}
	else if(line.strip_all_whitespaces().matches(identifier))
	{
	    if (was)
  	    {
		printerror("Fingerprint line without a Version found!",lines);
		ret=false;
	    }
	    was = true;
	    fp_line = line.substring (identifier.length()+2, 0);
	}
	else
	{
	    printerror("Invalid line found!",lines);
	    ret = false;
	}

        if ( (!ret || lenge<0) && !polite)
	{
	    datei.Close();
	    return false;
	}
	if ( !was ) continue;

	switch (fpt)
	{
	    case FP_Conf :
		cfp = new fp_conf(version);
		break;
	    case FP_Return :
		cfp = new fp_return(version);
		break;
	    case FP_Rfc :
		cfp = new fp_rfc(version);
		break;
	    case FP_Pop3 :
	    case FP_Ftp  :
	    case FP_Imap :
	    case FP_Base :
		throw x_implemented("Trying to load non-existend fingerprint types",__FLF__);
		break;
	}

        fps = gen_fp_data (cfp, fp_line);
	if (fps <= 0)	// In this case, there was an error during generation
	{
	    printerror("Invalid FP_CONF line found",lines);
	    ret = false;//if (polite) { continue; } else { return false; }
	}

	if ( debuglevel > 1 )
	{
	    cout << "Found this fingerprint : " << endl;
	    switch (fpt)
	    {
		case FP_Conf :
		    cout << *((fp_conf*)cfp) << endl;
		    break;
		case FP_Return :
		    cout << *((fp_return*)cfp) << endl;
		    break;
		case FP_Rfc :
		    cout << *((fp_rfc*)cfp) << endl;
		    break;
		case FP_Pop3 :
		case FP_Ftp  :
		case FP_Imap :
		case FP_Base :
		    throw x_implemented("Trying to load non-existend fingerprint types",__FLF__);
		    break;
	    }
	}
	
	add_fp(*cfp,fpt);

	delete cfp;
	if ( check_leng )
	{
	    try
	    {
		lenge = check_lenge (lenge, fps, lines,polite );
	    } 
	    catch(...)
	    {
		ret = false;
	    }
	}
    }
    datei.Close();
    return true;
}/*}}}*/

uint32 scanner_server::check_lenge (uint32 lenge, uint32 fps, uint32 lines ,BOOL polite)/*{{{*/
{
    if (lenge == 0)
    {
	lenge = fps;
    }
    else if (lenge != fps)
    {
	printerror( "FP Size does not match previously ones.", lines);
	throw(-1);
    }

    return lenge;
}/*}}}*/

void scanner_server::printerror( const mstring & error, uint32 line )/*{{{*/
{
    cout << "Error in line " << line << " : " << error;
    if (polite)
    {
	cout << " ( --polite set, ignoring ) " << endl;
    }
    else
    {
	cout << endl;
    }
}/*}}}*/

sint32 scanner_server::gen_fp_data (fp * cfp, mstring & fp)/*{{{*/
{
    uint32 nk=0;
    uint32 fps=0;
    uint32 sfp=0;
    char * end=NULL;
    mstring num;

    do
    {
	nk = fp.lsearch (",");
	num = fp.substring (1, nk - 1);
	fp = fp.substring (nk + 1, 0);
	fps++;
	sfp = strtol (num, &end, 10);
	if ( end != num[num.length()] )
	{
	    return -1;
	}
	// generate a more elaborate version of this stuff here 
	cfp->add (sfp);
    }
    while (fp.lsearch (",") > 0);

    sfp = strtol (fp, NULL, 10);
    cfp->add (sfp);
    return fps;
}/*}}}*/

mstring scanner_server::get_next_line ( mfile& datei)/*{{{*/
{
    mstring line;
    int rpos;
    line = datei.ReadLine().strip_whitespaces();
    rpos = line.lsearch("#");
    if (rpos == 1) return mstring("");
    line = line.substring(1,rpos-1);
    return line.strip_whitespaces();
}/*}}}*/
/**
 *$Log: scanner_server.cpp,v $
 *Revision 1.13  2002/11/25 23:33:35  plasmahh
 *changed a bit stuff and new fingerprints and and and ;)
 *
 *Revision 1.12  2002/11/06 18:49:45  plasmahh
 *some fixes and stuff... ftp implemented...
 *
 *Revision 1.11  2002/11/05 20:56:27  plasmahh
 *new stuff, now ftp should work quite good
 *
 *Revision 1.10  2002/11/04 23:45:52  plasmahh
 *first addition, implementation and changes for ftp fingerprinting
 *rfc fp fix still mising !
 *
 *Revision 1.9  2002/10/21 08:38:29  plasmahh
 *begin cleanup and structure changes for smtpmap 0.8-stable
 *
 *Revision 1.8  2002/10/01 20:59:02  plasmahh
 *some new structures, to maintain multiple protocols bette
 *
 *Revision 1.7  2002/08/12 13:13:05  plasmahh
 *removed some memory leaks
 *
 *Revision 1.6  2002/08/08 22:13:46  plasmahh
 *fixed bug with really fast connecting
 *
 *Revision 1.5  2002/08/08 21:57:57  plasmahh
 *DEBUG !!!
 *
 *Revision 1.4  2002/08/08 21:47:28  plasmahh
 *more debug
 *
 *Revision 1.3  2002/06/18 13:42:39  plasmahh
 *fixed timeout behaviour
 *new fingerprints added
 *improved documentation
 *
 *Revision 1.2  2002/06/17 20:41:10  plasmahh
 *fixed little bug
 *
 *Revision 1.1  2002/06/17 12:23:48  plasmahh
 *fixed timeout bugs
 *
 */
